--[[

RandomCompanion
Picks a random vanity pet or mount and automatically chooses the best type of mount for the job.

Usage:
Bind a key from the keybinding menu or...
"/rc mount" to choose a random mount
"/rc mount ground" to choose a random ground mount in a flyable zone
"/rc pet" to choose a random vanity pet
"/rc autorecall" to toggle automatically recalling your vanity pet after resurrecting, changing zones, or taking flight paths
"/rc randomrecall" to toggle recalling a random vanity pet after resurrecting, changing zones, or taking flight paths
"/rc dismiss" to dismiss your current vanity pet
"/rc autodismiss" to toggle automatically dismissing your vanity pet when you are stealthed and flagged for PVP

TODO:
Button that shows your current pet/mount or a ? if you don't have one out
Shift out of ghostwolf and druid forms before mounting.
Check if Frozen Sea needs to be added to Northrend zones
Headless horseman's mount can not be used while swimming in a flyable area
Add options to AddOns panel
Cache snowball status

Changes:
1.7.3
Lowered the amount of time between moving the weight slider and saving the weight.
Updated for version 3.3.

1.7.2
When you are in a vehicle, you will exit the vehicle when you trying to mount.
Added macro conditional parsing to /rc. Example: "/rc [button:1] mount; [button:2] pet" will summon your mount when you left-click the macro, or your pet when you right-click.

1.7.1
Fixed a problem with initialization that was introduced in 1.7.
Fixed a problem where randomly summoning the same vanity pet that you already have out will instead dismiss your current pet.
Added invisibility to the list of buffs which will stop AutoRecall.

1.7
Consolidated all /commands under /rc or /randomcompanion. Old /commands will remain in place for now for backward compatibility. Type "/rc" for a full list of available commands.
Added "/rc autodismiss" to toggle automatically dismissing your vanity pet when you are stealthed and flagged for PVP.
Added "/rc randomrecall" to toggle recalling a random vanity pet after resurrecting, changing zones, or taking flight paths.
Fixed some oddities with AutoRecall.
AutoRecall (and RandomRecall) now recalls your vanity pet when you log in.
Added support for the Onyxian Drake.

1.6.2
Added localization for Krasus' Landing and Underbelly for the German client. Thanks, Herekren.
Added localization for Krasus' Landing for the French and Russian clients. Still need Underbelly translations for these clients.
Added support for the Rusted and Ironbound Proto-Drakes.
Updated name of Headless Horseman's Mount because it changed slightly in 3.2.
Removed Dalaran as special flyable zone because IsFlyableArea() finally works as expected there.
Wintergrasp is now flyable when a battle is not in progress.

1.6.1
Fixed a bug with the weight UI that happened after changing zones

1.6
UI to disable mounts/pets that you don't like or choose your favorite mounts/pets more often has finally been added! Just open your Mount/Pet window and adjust the slider.
Black Qiraji Battle Tank should now work properly both inside and outside of Ahn'Qiraj.

1.5.1
Fixed a bug that detects other mounts as swimmers if you have a sea turtle.

1.5
RandomCompanion will now choose a swimming mount (the new sea turtle) or a ground mount if you are swimming.
Added "/rpet dismiss" and "/pdismiss" to dismiss your current vanity pet.
Vanity pets are automatically re-summoned after losing them for any reason. "/rpet autorecall" to toggle this feature.

1.4
Added Russian localization for Dragonblight.
Fixed a bug with flyable detection in the sewer pipe section of Dalaran.
Fixed a bug with flyable detection in part of Krasus' Landing in Dalaran.
Added flyable detection to some areas on the outskirts of Dalaran. It's not perfect, but it's better than it used to be.
Added /rcreload in case you ever want to re-initialize for some reason.
Added macro conditional parsing such as [modifier:alt] or [combat] to /rmount.
Added checks to make sure you have engineering for the engineering mounts and tailoring for the tailoring mounts. Does not check skill level, so if you re-learn your old tradeskill, your not-yet-usable mounts may be chosen.
Vanity pets are automatically re-summoned after resurrecting, changing zones, or taking flight paths. "/rpet autorecall" to toggle this feature.
LOTS of new debug messages (/rcdebug to enable), so if you're having any sort of problems, turn on debug mode and report them to http://wow.curse.com/downloads/wow-addons/details/randomcompanion.aspx
Added a check for snowballs before summoning winter pets (Winter Reindeer, etc.)

1.3
Fixed bug mount prioritization in AQ40 - May not work properly in non-English clients yet.
Changed the German localization to support an oddity with the Turbogetriebene Flugmaschine.
Added /rcstatus to get RandomCompanion status and /rcdebug to toggle debug mode.
Fixed a bug with scaling flying mounts not getting added to the slow flyers list.
Uncached mounts and pets should be handled a lot better.
Fixed a bug with flyable detection inside Dalaran.

]]--

local VERSION = GetAddOnMetadata("RandomCompanion", "Version");

RandomCompanion = {
	mounts = {
		unweighted = {
			ground = {};
			flying = {};
			swiftground = {};
			swiftflying = {};
			extremeflying = {};
			swimmers = {};
			bugs = {};
		};
		weighted = {
			ground = {};
			flying = {};
			swiftground = {};
			swiftflying = {};
			extremeflying = {};
			swimmers = {};
			bugs = {};
		};
	};
	critters = {
		weighted = {};
		unweighted = {};
		winter = {};
	};
	Engineering = 0;
	Tailoring = 0;
};

local function debugmsg(msg, debuglevel)
	if tonumber(debuglevel) then
		local debuglevel = tonumber(debuglevel);
	else	
		local debuglevel = 1;
	end
	if RandomCompanion_settings.DEBUG and (debuglevel >= tonumber(RandomCompanion_settings.DEBUGLevel)) then
		DEFAULT_CHAT_FRAME:AddMessage('RandomCompanion Debug[' .. debuglevel .. ']: ' .. msg);
	end
end

function RandomCompanion.Event(self, event, ...)
	if event == "PLAYER_LOGIN" then
		RandomCompanion.Initialize(true);
	elseif event == "COMPANION_LEARNED" then
		RandomCompanion.Initialize(false);
	elseif event == "COMPANION_UPDATE" then
		RandomCompanion.SetActiveCompanions();
	elseif event == "PLAYER_ENTERING_WORLD" or event == "PLAYER_CONTROL_GAINED" or event == "PLAYER_UNGHOST" or event == "PLAYER_ALIVE" then
		RandomCompanion.RecallCritter();
	else
		debugmsg("Unknown event triggered.");
	end
end

function RandomCompanion.Help()
	--For backwards compatibility with old slash commands
	RandomCompanion.Slash("help");
end

function RandomCompanion.Initialize(showstatus)
	RC_REQUIRE_RELOAD = false;
	
	if not RandomCompanion_settings then
		RandomCompanion_settings = {};
	end
	
	if not RandomCompanion_settings[GetRealmName()] then
		RandomCompanion_settings[GetRealmName()] = {};
	end
	
	if not RandomCompanion_settings[GetRealmName()][UnitName("player")] then
		RandomCompanion_settings[GetRealmName()][UnitName("player")] = {};
	end
	
	if not RandomCompanion_settings.DEBUG then
		RandomCompanion_settings.DEBUG = false;
	end
	
	if RandomCompanion_settings.DEBUGLevel == nil then
		RandomCompanion_settings.DEBUGLevel = 1;
	end
	
	if RandomCompanion_settings.AutoRecall == nil then
		RandomCompanion_settings.AutoRecall = true;
		if (RandomCompanion_settings.AutoRecall or RandomCompanion_settings.RandomRecall or RandomCompanion_settings.AutoDismiss) and RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter then
			RandomCompanion.OnUpdateTime = GetTime() + 2;
			RandomCompanion.EventFrame:SetScript("OnUpdate", RandomCompanion.OnUpdate);
		else
			RandomCompanion.EventFrame:SetScript("OnUpdate", nil);
		end
	end
	
	if RandomCompanion_settings.AutoDismiss == nil then
		RandomCompanion_settings.AutoDismiss = true;
		if (RandomCompanion_settings.AutoRecall or RandomCompanion_settings.RandomRecall or RandomCompanion_settings.AutoDismiss) and RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter then
			RandomCompanion.OnUpdateTime = GetTime() + 2;
			RandomCompanion.EventFrame:SetScript("OnUpdate", RandomCompanion.OnUpdate);
		else
			RandomCompanion.EventFrame:SetScript("OnUpdate", nil);
		end
	end
	
	if RandomCompanion_settings.RandomRecall == nil then
		RandomCompanion_settings.RandomRecall = false;
	end
	
	if not RandomCompanion_settings[GetRealmName()] then
		RandomCompanion_settings[GetRealmName()] = {};
	end
	
	if not RandomCompanion_settings[GetRealmName()][UnitName("player")] then
		RandomCompanion_settings[GetRealmName()][UnitName("player")] = {};
	end
	
	if not RandomCompanion_settings[GetRealmName()][UnitName("player")].weights then
		RandomCompanion_settings[GetRealmName()][UnitName("player")].weights = {};
	end
	
	local scalingmounts = {
		ground = {};
		flying = {};
	};
	
	RandomCompanion.mounts = {
		unweighted = {
			ground = {};
			flying = {};
			swiftground = {};
			swiftflying = {};
			extremeflying = {};
			swimmers = {};
			bugs = {};
		};
		weighted = {
			ground = {};
			flying = {};
			swiftground = {};
			swiftflying = {};
			extremeflying = {};
			swimmers = {};
			bugs = {};
		};
	};
	RandomCompanion.critters = {
		weighted = {};
		unweighted = {};
		winter = {};
	};
	
	if not RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter then
		RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter = nil;
		RandomCompanion.SetActiveCompanions();
	end
	
	RandomCompanion.Engineering = 0;
	RandomCompanion.Tailoring = 0;
	
	CreateFrame("GameTooltip", "RandomCompanionScanningTooltip", UIParent, "GameTooltipTemplate");
	RandomCompanionScanningTooltip:AddFontStrings(
		RandomCompanionScanningTooltip:CreateFontString( "$parentTextLeft1", nil, "GameTooltipText" ),
		RandomCompanionScanningTooltip:CreateFontString( "$parentTextRight1", nil, "GameTooltipText" )
	);
	
	for i = 1, GetNumSkillLines() do
		local skillName, isHeader, isExpanded, skillRank, numTempPoints, skillModifier, skillMaxRank, isAbandonable, stepCost, rankCost, minLevel, skillCostType, skillDescription = GetSkillLineInfo(i)
		if (skillName == RC_STRING_ENGINEERING) then
			RandomCompanion.Engineering = skillRank;
			debugmsg("Found engineering skill " .. skillRank, 8);
		end
		if (skillName == RC_STRING_TAILORING) then
			RandomCompanion.Tailoring = skillRank;
			debugmsg("Found tailoring skill " .. skillRank, 8);
		end
	end

	for index = 1, GetNumCompanions("MOUNT") do
		local aq = false;
		local swimmer = false;
		local outland = false;
		local northrend = false;
		local very = false;
		local extremely = false;
		local fast = false;
		local engineering = false;
		local tailoring = false;
		local creatureid, name, spellid, icon, active = GetCompanionInfo("MOUNT", index);
		
		if name then
			GameTooltip_SetDefaultAnchor(RandomCompanionScanningTooltip, UIParent) 
			RandomCompanionScanningTooltip:SetHyperlink("spell:"..spellid)
			
			for i = 1, RandomCompanionScanningTooltip:NumLines() do
				local text = _G["RandomCompanionScanningTooltipTextLeft"..i]:GetText();
				
				if string.find(text, RC_STRING_AHN_QIRAJ) then
					aq = true;
				end
				if string.find(text, RC_STRING_SWIMMER) then
					swimmer = true;
				end
				if string.find(text, RC_STRING_OUTLAND) then
					outland = true;
				end
				if string.find(text, RC_STRING_NORTHREND) then
					northrend = true;
				end
				if string.find(text, RC_STRING_VERY) then
					very = true;
				end
				if string.find(text, RC_STRING_EXTREMELY) then
					extremely = true;
				end
				if string.find(text, RC_STRING_FAST) then
					fast = true;
				end
				if string.find(string.lower(text), string.lower(RC_STRING_ENGINEERING)) then
					engineering = true;
				end
				if string.find(string.lower(text), string.lower(RC_STRING_TAILORING)) then
					tailoring = true;
				end
			end
			
			if not RandomCompanion_settings[GetRealmName()][UnitName("player")].weights[name] then
				RandomCompanion.SetWeight(name, 1);
			end
			
			local weight = RandomCompanion.GetWeight(name);
			
			if (not engineering or RandomCompanion.Engineering > 0) and (not tailoring or RandomCompanion.Tailoring > 0) then
				if name == RC_STRING_BLIZZARD_BEAR then
					--This mount has to be handled separately because it scales based on skill
					--Thanks to Huntrah-Mal'ganis for helping to test this
					debugmsg("OMG! You have the Blizzcon Bear! AWESOME!", 5);
					local nextindex = table.getn(scalingmounts.ground) + 1;
					scalingmounts.ground[nextindex] = {
						creatureid = creatureid;
						name = name;
						index = index;
						spellid = spellid;
						icon = icon;
						active = active;
					}
				elseif name == RC_STRING_HORSEMAN_MOUNT then
					--This mount has to be handled separately because it scales based on location and skill
					--Thanks to Huntrah-Mal'ganis for helping to test this
					debugmsg("OMG! You have the Horseman's Mount! AWESOME!", 5);
					local nextindex = table.getn(scalingmounts.flying) + 1;
					scalingmounts.flying[nextindex] = {
						creatureid = creatureid;
						name = name;
						index = index;
						spellid = spellid;
						icon = icon;
						active = active;
					}
					local nextindex = table.getn(scalingmounts.ground) + 1;
					scalingmounts.ground[nextindex] = {
						creatureid = creatureid;
						name = name;
						index = index;
						spellid = spellid;
						icon = icon;
						active = active;
					}
				elseif name == RC_STRING_DK_FLYER then
					--This mount has to be handled separately because it scales based on skill
					debugmsg("OMG! You have the Deathknight Flying Mount! AWESOME!", 5);
					local nextindex = table.getn(scalingmounts.flying) + 1;
					scalingmounts.flying[nextindex] = {
						creatureid = creatureid;
						name = name;
						index = index;
						spellid = spellid;
						icon = icon;
						active = active;
					}
				elseif swimmer then
					local nextindex = table.getn(RandomCompanion.mounts.unweighted.swimmers) + 1;
					RandomCompanion.mounts.unweighted.swimmers[nextindex] = {
						creatureid = creatureid;
						name = name;
						index = index;
						spellid = spellid;
						icon = icon;
						active = active;
					}
				elseif name == RC_STRING_BLACK_QIRAJI_BATTLE_TANK then
					debugmsg("OMG! You have the Black Bug Mount! AWESOME!", 5);
					local nextindex = table.getn(RandomCompanion.mounts.unweighted.swiftground) + 1;
					RandomCompanion.mounts.unweighted.swiftground[nextindex] = {
						creatureid = creatureid;
						name = name;
						index = index;
						spellid = spellid;
						icon = icon;
						active = active;
					}
					local nextindex = table.getn(RandomCompanion.mounts.unweighted.bugs) + 1;
					RandomCompanion.mounts.unweighted.bugs[nextindex] = {
						creatureid = creatureid;
						name = name;
						index = index;
						spellid = spellid;
						icon = icon;
						active = active;
					}
				elseif aq then
					local nextindex = table.getn(RandomCompanion.mounts.unweighted.bugs) + 1;
					RandomCompanion.mounts.unweighted.bugs[nextindex] = {
						creatureid = creatureid;
						name = name;
						index = index;
						spellid = spellid;
						icon = icon;
						active = active;
					}
				--310% speed mounts have to be specified because most of them do not have special text to indicate they are faster than normal
				elseif (extremely and fast and (outland or northrend)) or name == RC_STRING_ASHES_OF_ALAR or name == RC_STRING_BLACK_PROTO_DRAKE or name == RC_STRING_PLAGUED_PROTO_DRAKE or name == RC_STRING_VIOLET_PROTO_DRAKE or name == RC_STRING_RUSTED_PROTO_DRAKE or name == RC_STRING_IRONBOUND_PROTO_DRAKE or name == RC_STRING_ONYXIAN_DRAKE then
					local nextindex = table.getn(RandomCompanion.mounts.unweighted.extremeflying) + 1;
					RandomCompanion.mounts.unweighted.extremeflying[nextindex] = {
						creatureid = creatureid;
						name = name;
						index = index;
						spellid = spellid;
						icon = icon;
						active = active;
					}
				elseif very and fast and (outland or northrend) then
					local nextindex = table.getn(RandomCompanion.mounts.unweighted.swiftflying) + 1;
					RandomCompanion.mounts.unweighted.swiftflying[nextindex] = {
						creatureid = creatureid;
						name = name;
						index = index;
						spellid = spellid;
						icon = icon;
						active = active;
					}
				elseif very and fast and not outland and not northrend then
					local nextindex = table.getn(RandomCompanion.mounts.unweighted.swiftground) + 1;
					RandomCompanion.mounts.unweighted.swiftground[nextindex] = {
						creatureid = creatureid;
						name = name;
						index = index;
						spellid = spellid;
						icon = icon;
						active = active;
					}
				elseif not very and not fast and (outland or northrend) then
					local nextindex = table.getn(RandomCompanion.mounts.unweighted.flying) + 1;
					RandomCompanion.mounts.unweighted.flying[nextindex] = {
						creatureid = creatureid;
						name = name;
						index = index;
						spellid = spellid;
						icon = icon;
						active = active;
					}
				elseif not very and not fast and not outland and not northrend then
					local nextindex = table.getn(RandomCompanion.mounts.unweighted.ground) + 1;
					RandomCompanion.mounts.unweighted.ground[nextindex] = {
						creatureid = creatureid;
						name = name;
						index = index;
						spellid = spellid;
						icon = icon;
						active = active;
					}
				else
					-- This should never happen, but just in case...
					local options = "";
					if very then
						options = options .. " very";
					end
					if extremely then
						options = options .. " extremely";
					end
					if fast then
						options = options .. " fast";
					end
					if outland then
						options = options .. " outland";
					end
					if northrend then
						options = options .. " northrend";
					end
					if options == "" then
						options = " none";
					end
					DEFAULT_CHAT_FRAME:AddMessage("RandomCompanion " .. VERSION .. ":" .. name .. " mount type could not be detected. (spellid:" .. spellid .. " options:" .. options .. ")");
					DEFAULT_CHAT_FRAME:AddMessage("Please leave a comment about this at http://wow.curse.com/downloads/wow-addons/details/randomcompanion.aspx");
				end
			else
				debugmsg(name .. " requires a tradeskill that you no longer have.", 8);
			end
			
			if weight ~= 1 then
				debugmsg("Mount " .. name .. " has non-default weight: " .. weight, 8);
			end
			
			if weight > 0 then
				for counter = 1, weight do
					if name == RC_STRING_HORSEMAN_MOUNT or name == RC_STRING_BLIZZARD_BEAR or name == RC_STRING_DK_FLYER then
						--do nothing here because these get added in later
					elseif swimmer then
						local nextindex = table.getn(RandomCompanion.mounts.weighted.swimmers) + 1;
						RandomCompanion.mounts.weighted.swimmers[nextindex] = {
							creatureid = creatureid;
							name = name;
							index = index;
							spellid = spellid;
							icon = icon;
							active = active;
						}
					elseif name == RC_STRING_BLACK_QIRAJI_BATTLE_TANK then
						local nextindex = table.getn(RandomCompanion.mounts.weighted.swiftground) + 1;
						RandomCompanion.mounts.weighted.swiftground[nextindex] = {
							creatureid = creatureid;
							name = name;
							index = index;
							spellid = spellid;
							icon = icon;
							active = active;
						}
						local nextindex = table.getn(RandomCompanion.mounts.weighted.bugs) + 1;
						RandomCompanion.mounts.weighted.bugs[nextindex] = {
							creatureid = creatureid;
							name = name;
							index = index;
							spellid = spellid;
							icon = icon;
							active = active;
						}
					elseif aq then
						local nextindex = table.getn(RandomCompanion.mounts.weighted.bugs) + 1;
						RandomCompanion.mounts.weighted.bugs[nextindex] = {
							creatureid = creatureid;
							name = name;
							index = index;
							spellid = spellid;
							icon = icon;
							active = active;
						}
					--310% speed mounts have to be specified because most of them do not have special text to indicate they are faster than normal
					elseif (extremely and fast and (outland or northrend)) or name == RC_STRING_ASHES_OF_ALAR or name == RC_STRING_BLACK_PROTO_DRAKE or name == RC_STRING_PLAGUED_PROTO_DRAKE or name == RC_STRING_VIOLET_PROTO_DRAKE or name == RC_STRING_RUSTED_PROTO_DRAKE or name == RC_STRING_IRONBOUND_PROTO_DRAKE or name == RC_STRING_ONYXIAN_DRAKE then
						local nextindex = table.getn(RandomCompanion.mounts.weighted.extremeflying) + 1;
						RandomCompanion.mounts.weighted.extremeflying[nextindex] = {
							creatureid = creatureid;
							name = name;
							index = index;
							spellid = spellid;
							icon = icon;
							active = active;
						}
					elseif very and fast and (outland or northrend) then
						local nextindex = table.getn(RandomCompanion.mounts.weighted.swiftflying) + 1;
						RandomCompanion.mounts.weighted.swiftflying[nextindex] = {
							creatureid = creatureid;
							name = name;
							index = index;
							spellid = spellid;
							icon = icon;
							active = active;
						}
					elseif very and fast and not outland and not northrend then
						local nextindex = table.getn(RandomCompanion.mounts.weighted.swiftground) + 1;
						RandomCompanion.mounts.weighted.swiftground[nextindex] = {
							creatureid = creatureid;
							name = name;
							index = index;
							spellid = spellid;
							icon = icon;
							active = active;
						}
					elseif not very and not fast and (outland or northrend) then
						local nextindex = table.getn(RandomCompanion.mounts.weighted.flying) + 1;
						RandomCompanion.mounts.weighted.flying[nextindex] = {
							creatureid = creatureid;
							name = name;
							index = index;
							spellid = spellid;
							icon = icon;
							active = active;
						}
					elseif not very and not fast and not outland and not northrend then
						local nextindex = table.getn(RandomCompanion.mounts.weighted.ground) + 1;
						RandomCompanion.mounts.weighted.ground[nextindex] = {
							creatureid = creatureid;
							name = name;
							index = index;
							spellid = spellid;
							icon = icon;
							active = active;
						}
					end
				end
			end
		else
			--This seems to happen when the companion is not in the cache
			RC_REQUIRE_RELOAD = true;
			debugmsg(index .. " index mount has no name?");
		end
	end
	
	RandomCompanionScanningTooltip:Hide()
	
	--Add in the scaling flying mounts
	debugmsg("Processing scaling flying mounts", 2);
	for key, mount in pairs(scalingmounts.flying) do
		debugmsg(mount.name .. " is a scaling flyer", 2);
		if table.getn(RandomCompanion.mounts.unweighted.swiftflying) > 0 then
			debugmsg("Adding " .. mount.name .. " to unweighted swiftflying.", 2);
			local nextindex = table.getn(RandomCompanion.mounts.unweighted.swiftflying) + 1;
			RandomCompanion.mounts.unweighted.swiftflying[nextindex] = {
				creatureid = mount.creatureid;
				name = mount.name;
				index = mount.index;
				spellid = mount.spellid;
				icon = mount.icon;
				active = mount.active;
			};
		end
		if table.getn(RandomCompanion.mounts.unweighted.flying) > 0 or mount.name == RC_STRING_DK_FLYER then
			debugmsg("Adding " .. mount.name .. " to unweighted flying.", 2);
			local nextindex = table.getn(RandomCompanion.mounts.unweighted.flying) + 1;
			RandomCompanion.mounts.unweighted.flying[nextindex] = {
				creatureid = mount.creatureid;
				name = mount.name;
				index = mount.index;
				spellid = mount.spellid;
				icon = mount.icon;
				active = mount.active;
			};
		end
		
		local weight = RandomCompanion.GetWeight(mount.name);
		
		if weight > 0 then
			for counter = 1, weight do
				if table.getn(RandomCompanion.mounts.weighted.swiftflying) > 0 then
					debugmsg("Adding " .. mount.name .. " to weighted swiftflying.", 2);
					local nextindex = table.getn(RandomCompanion.mounts.weighted.swiftflying) + 1;
					RandomCompanion.mounts.weighted.swiftflying[nextindex] = {
						creatureid = mount.creatureid;
						name = mount.name;
						index = mount.index;
						spellid = mount.spellid;
						icon = mount.icon;
						active = mount.active;
					}
				end
				if table.getn(RandomCompanion.mounts.weighted.flying) > 0 or mount.name == RC_STRING_DK_FLYER then
					debugmsg("Adding " .. mount.name .. " to weighted flying.", 2);
					local nextindex = table.getn(RandomCompanion.mounts.weighted.flying) + 1;
					RandomCompanion.mounts.weighted.flying[nextindex] = {
						creatureid = mount.creatureid;
						name = mount.name;
						index = mount.index;
						spellid = mount.spellid;
						icon = mount.icon;
						active = mount.active;
					}
				end
			end
		end
	end
	
	--Add in the scaling ground mounts
	for key, mount in pairs(scalingmounts.ground) do
		if table.getn(RandomCompanion.mounts.unweighted.swiftground) > 0 then
			local nextindex = table.getn(RandomCompanion.mounts.unweighted.swiftground) + 1;
			RandomCompanion.mounts.unweighted.swiftground[nextindex] = {
				creatureid = mount.creatureid;
				name = mount.name;
				index = mount.index;
				spellid = mount.spellid;
				icon = mount.icon;
				active = mount.active;
			};
		end
		local nextindex = table.getn(RandomCompanion.mounts.unweighted.ground) + 1;
		RandomCompanion.mounts.unweighted.swiftground[nextindex] = {
			creatureid = mount.creatureid;
			name = mount.name;
			index = mount.index;
			spellid = mount.spellid;
			icon = mount.icon;
			active = mount.active;
		};
		
		local weight = RandomCompanion.GetWeight(mount.name);
		
		if weight > 0 then
			for counter = 1, weight do
				if table.getn(RandomCompanion.mounts.weighted.swiftground) > 0 then
					local nextindex = table.getn(RandomCompanion.mounts.weighted.swiftground) + 1;
					RandomCompanion.mounts.weighted.swiftground[nextindex] = {
						creatureid = mount.creatureid;
						name = mount.name;
						index = mount.index;
						spellid = mount.spellid;
						icon = mount.icon;
						active = mount.active;
					}
				end
				local nextindex = table.getn(RandomCompanion.mounts.weighted.ground) + 1;
				RandomCompanion.mounts.weighted.ground[nextindex] = {
					creatureid = mount.creatureid;
					name = mount.name;
					index = mount.index;
					spellid = mount.spellid;
					icon = mount.icon;
					active = mount.active;
				}
			end
		end
	end
	
	for index = 1, GetNumCompanions("CRITTER") do
		local creatureid, name, spellid, icon, active = GetCompanionInfo("CRITTER", index);
		if name then
			if not RandomCompanion_settings[GetRealmName()][UnitName("player")].weights[name] then
				RandomCompanion.SetWeight(name, 1);
			end
			local weight = RandomCompanion.GetWeight(name);
			
			local nextindex = table.getn(RandomCompanion.critters.unweighted) + 1;
			RandomCompanion.critters.unweighted[nextindex] = {
				creatureid = creatureid;
				name = name;
				index = index;
				spellid = spellid;
				icon = icon;
				active = active;
			}
			
			if weight ~= 1 then
				debugmsg("Critter " .. name .. " has non-default weight: " .. weight, 2);
			end
			
			if weight > 0 then
				for counter = 1, weight do
					local nextindex = table.getn(RandomCompanion.critters.weighted) + 1;
					RandomCompanion.critters.weighted[nextindex] = {
						creatureid = creatureid;
						name = name;
						index = index;
						spellid = spellid;
						icon = icon;
						active = active;
					}
				end
			end
			
			--Check to see if this critter requires a snowball to summon
			GameTooltip_SetDefaultAnchor(RandomCompanionScanningTooltip, UIParent) 
			RandomCompanionScanningTooltip:SetHyperlink("spell:"..spellid)
			
			for i = 1, RandomCompanionScanningTooltip:NumLines() do
				local text = _G["RandomCompanionScanningTooltipTextLeft"..i]:GetText();
				
				if string.find(string.lower(text), string.lower(RC_STRING_SNOWBALL)) then
					RandomCompanion.critters.winter[index] = true;
					debugmsg(name .. " (index: " .. index .. ") is a winter pet that requires a snowball to summon.", 3);
					break;
				end
			end
			
			RandomCompanionScanningTooltip:Hide()
		else
			--This seems to happen when the companion is not in the cache
			RC_REQUIRE_RELOAD = true;
			debugmsg(index .. " index critter has no name?");
		end
	end
	
	RandomCompanion.SetActiveCompanions();
	
	if showstatus then
		RandomCompanion.Status();
	end
end

function RandomCompanion.Debug()
	--For backwards compatibility with old slash commands
	RandomCompanion.Slash("debug");
end

function RandomCompanion.Status()
	local loadmessage = "";
	
	if table.getn(RandomCompanion.mounts.unweighted.extremeflying) > 0 then
		if loadmessage ~= "" then
			loadmessage = loadmessage .. ", ";
		end
		loadmessage = loadmessage .. table.getn(RandomCompanion.mounts.unweighted.extremeflying) .. " extreme flyers";
	end
	if table.getn(RandomCompanion.mounts.unweighted.swiftflying) > 0 then
		if loadmessage ~= "" then
			loadmessage = loadmessage .. ", ";
		end
		loadmessage = loadmessage .. table.getn(RandomCompanion.mounts.unweighted.swiftflying) .. " swift flyers";
	end
	if table.getn(RandomCompanion.mounts.unweighted.flying) > 0 then
		if loadmessage ~= "" then
			loadmessage = loadmessage .. ", ";
		end
		loadmessage = loadmessage .. table.getn(RandomCompanion.mounts.unweighted.flying) .. " slow flyers";
	end
	if table.getn(RandomCompanion.mounts.unweighted.swiftground) > 0 then
		if loadmessage ~= "" then
			loadmessage = loadmessage .. ", ";
		end
		loadmessage = loadmessage .. table.getn(RandomCompanion.mounts.unweighted.swiftground) .. " swift grounders";
	end
	if table.getn(RandomCompanion.mounts.unweighted.ground) > 0 then
		if loadmessage ~= "" then
			loadmessage = loadmessage .. ", ";
		end
		loadmessage = loadmessage .. table.getn(RandomCompanion.mounts.unweighted.ground) .. " slow grounders";
	end
	if table.getn(RandomCompanion.mounts.unweighted.bugs) > 0 then
		if loadmessage ~= "" then
			loadmessage = loadmessage .. ", ";
		end
		loadmessage = loadmessage .. table.getn(RandomCompanion.mounts.unweighted.bugs) .. " bugs";
	end
	if table.getn(RandomCompanion.mounts.unweighted.swimmers) > 0 then
		if loadmessage ~= "" then
			loadmessage = loadmessage .. ", ";
		end
		loadmessage = loadmessage .. table.getn(RandomCompanion.mounts.unweighted.swimmers) .. " swimmers";
	end
	if table.getn(RandomCompanion.critters.unweighted) > 0 then
		if loadmessage ~= "" then
			loadmessage = loadmessage .. ", ";
		end
		loadmessage = loadmessage .. table.getn(RandomCompanion.critters.unweighted) .. " critters";
	end
	
	if loadmessage ~= "" then
		loadmessage = " with " .. loadmessage .. ".";
	end
	
	if RandomCompanion_settings.RandomRecall then
		loadmessage = loadmessage .. " Vanity pets will be automatically randomly re-summoned when possible.";
	elseif RandomCompanion_settings.AutoRecall then
		loadmessage = loadmessage .. " Vanity pets will be automatically re-summoned when possible.";
	end
	
	if (RandomCompanion_settings.AutoRecall or RandomCompanion_settings.RandomRecall or RandomCompanion_settings.AutoDismiss) and RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter then
		RandomCompanion.OnUpdateTime = GetTime() + 2;
		RandomCompanion.EventFrame:SetScript("OnUpdate", RandomCompanion.OnUpdate);
	else
		RandomCompanion.EventFrame:SetScript("OnUpdate", nil);
	end
	
	if RandomCompanion_settings.AutoDismiss then
		loadmessage = loadmessage .. " Vanity pets will be automatically dismissed when entering stealth while flagged for PVP.";
		if (RandomCompanion_settings.AutoRecall or RandomCompanion_settings.RandomRecall or RandomCompanion_settings.AutoDismiss) and RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter then
			RandomCompanion.OnUpdateTime = GetTime() + 2;
			RandomCompanion.EventFrame:SetScript("OnUpdate", RandomCompanion.OnUpdate);
		else
			RandomCompanion.EventFrame:SetScript("OnUpdate", nil);
		end
	end
	
	if RC_REQUIRE_RELOAD then
		loadmessage = loadmessage .. " RandomCompanion will reload due to uncached companion names.";
	end
	
	
	DEFAULT_CHAT_FRAME:AddMessage("RandomCompanion " .. VERSION .. " loaded" .. loadmessage);
end

function RandomCompanion.Mount(cmd)
	local flyable;
	
	if (cmd) then
		--Parse any macro conditionals such as [modifier:ctrl] or [combat]
		cmd = SecureCmdOptionParse(cmd);
	end
	
	if RC_REQUIRE_RELOAD then
		--Re-initialize if one of the pet or mount names was not available in the initial load
		debugmsg("Re-initializing RandomCompanion");
		RandomCompanion.Initialize();
	end
	
	if IsMounted() then
		Dismount();
	elseif CanExitVehicle() then
		VehicleExit();
	else
		if IsSwimming() then
			if table.getn(RandomCompanion.mounts.weighted.swimmers) > 0 then
				mountlist = "swimmers";
			elseif table.getn(RandomCompanion.mounts.weighted.swiftground) > 0 then
				mountlist = "swiftground";
			elseif table.getn(RandomCompanion.mounts.weighted.ground) > 0 then
				mountlist = "ground";
			else
				mountlist = nil;
			end
		elseif GetRealZoneText() == RC_STRING_TEMPLE_OF_AHN_QIRAJ then
			if table.getn(RandomCompanion.mounts.weighted.bugs) > 0 then
				mountlist = "bugs";
			elseif table.getn(RandomCompanion.mounts.weighted.swiftground) > 0 then
				mountlist = "swiftground";
			elseif table.getn(RandomCompanion.mounts.weighted.ground) > 0 then
				mountlist = "ground";
			else
				mountlist = nil;
			end
		elseif RandomCompanion.IsFlyableArea() and cmd ~= "ground" then
			if table.getn(RandomCompanion.mounts.weighted.extremeflying) > 0 then
				mountlist = "extremeflying";
			elseif table.getn(RandomCompanion.mounts.weighted.swiftflying) > 0 then
				mountlist = "swiftflying";
			elseif table.getn(RandomCompanion.mounts.weighted.flying) > 0 then
				mountlist = "flying";
			elseif table.getn(RandomCompanion.mounts.weighted.swiftground) > 0 then
				mountlist = "swiftground";
			elseif table.getn(RandomCompanion.mounts.weighted.ground) > 0 then
				mountlist = "ground";
			else
				mountlist = nil;
			end
		else
			if table.getn(RandomCompanion.mounts.weighted.swiftground) > 0 then
				mountlist = "swiftground";
			elseif table.getn(RandomCompanion.mounts.weighted.ground) > 0 then
				mountlist = "ground";
			else
				mountlist = nil;
			end
		end
		
		if mountlist then
			local mountindex = random(table.getn(RandomCompanion.mounts.weighted[mountlist]));
			debugmsg("Mounting " .. RandomCompanion.mounts.weighted[mountlist][mountindex].name .. " from " .. mountlist .." list. Index is " .. RandomCompanion.mounts.weighted[mountlist][mountindex].index, 10);
			CallCompanion("MOUNT", RandomCompanion.mounts.weighted[mountlist][mountindex].index);
		else
			if GetNumCompanions("MOUNT") > 0 then
				--We have mounts, but they aren't in the mount list. The most likely issue is that no mounts were in the cache when RandomCompanion.Initialize() ran, so let's run it again.
				if not RC_REINITIALIZED then
					debugmsg("Re-initializing because mount names weren't cached yet.");
					RC_REINITIALIZED = true; --We don't want an infinite loop, do we?
					RandomCompanion.Initialize();
					RandomCompanion.Mount(cmd);
				else
					RC_REINITIALIZED = nil;
				end
			else
				UIErrorsFrame:AddMessage("You have no mounts", 1.0, 0.0, 0.0, 53, 5);
			end
		end
	end
end

function RandomCompanion.IsFlyableArea()
	if IsFlyableArea() then
		if GetRealZoneText() == RC_STRING_BOREAN_TUNDRA or GetRealZoneText() == RC_STRING_CRYSTALSONG_FOREST or GetRealZoneText() == RC_STRING_DALARAN or GetRealZoneText() == RC_STRING_DRAGONBLIGHT or GetRealZoneText() == RC_STRING_GRIZZLY_HILLS or GetRealZoneText() == RC_STRING_HOWLING_FJORD or GetRealZoneText() == RC_STRING_ICECROWN or GetRealZoneText() == RC_STRING_SHOLAZAR_BASIN or GetRealZoneText() == RC_STRING_STORM_PEAKS or GetRealZoneText() == RC_STRING_WINTERGRASP or GetRealZoneText() == RC_STRING_ZUL_DRAK then
			coldflying, outOfMana = IsUsableSpell(RC_STRING_COLD_WEATHER_FLYING);
			
			if GetRealZoneText() == RC_STRING_WINTERGRASP then
				if GetWintergraspWaitTime() ~= nil and coldflying then
					return IsFlyableArea();
				else
					return false;
				end
			elseif coldflying then
				return IsFlyableArea();
			else
				return false;
			end
		else
			return IsFlyableArea();
		end
	else
		return IsFlyableArea();
	end
end

function RandomCompanion.Pet(cmd)
	if cmd and string.lower(cmd) == "autorecall" then
		--For backwards compatibility with old slash commands
		RandomCompanion.Slash(cmd);
	else
		if RC_REQUIRE_RELOAD then
			--Re-initialize if one of the pet or mount names was not available in the initial load
			debugmsg("Re-initializing RandomCompanion");
			RandomCompanion.Initialize();
		end
		
		if table.getn(RandomCompanion.critters.weighted) > 0 then
			local petindex = nil;
			local havesnowball = RandomCompanion.ScanForItem(RC_STRING_SNOWBALL);
			
			if havesnowball then
				debugmsg("Found a snowball in inventory. Winter pets may be summoned.", 3);
				petindex = random(table.getn(RandomCompanion.critters.weighted));
			else
				debugmsg("No snowballs in inventory. Winter pets may not be summoned.", 3);
				--Loop through 5 times trying to find a critter that doesn't require a snowball
				for counter = 1, 5 do 
					petindex = random(table.getn(RandomCompanion.critters.weighted));
					local companionindex = RandomCompanion.critters.weighted[petindex].index;
					if not RandomCompanion.critters.winter[companionindex] then
						debugmsg("Found a pet which does not require a snowball at index " .. RandomCompanion.critters.weighted[petindex].index, 3);
						break;
					else
						debugmsg("Skipping index " .. RandomCompanion.critters.weighted[petindex].index .. " (" .. RandomCompanion.critters.weighted[petindex].name .. ") because it requires a snowball.", 3);
					end
				end
			end
			
			--Only dismiss if there is an active critter and it's the same as the one that's about to be summoned, otherwise there are problems with RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter getting removed when you can't actually re-summon your pet, ie when dead
			if RandomCompanion.GetActiveCompanion("CRITTER") and RandomCompanion.GetActiveCompanion("CRITTER") == RandomCompanion.critters.weighted[petindex].index then
				DismissCompanion("CRITTER");
				debugmsg("Dismissing because critter is already active", 2);
			end
			
			debugmsg("Summoning " .. RandomCompanion.critters.weighted[petindex].name .. ". Index is " .. RandomCompanion.critters.weighted[petindex].index, 10);
			CallCompanion("CRITTER", RandomCompanion.critters.weighted[petindex].index);
		else
			if GetNumCompanions("CRITTER") > 0 then
				--We have pets, but they aren't in the pet list. The most likely issue is that no pets were in the cache when RandomCompanion.Initialize() ran, so let's run it again.
				if not RC_REINITIALIZED then
					debugmsg("Re-initializing because pet names weren't cached yet.");
					RC_REINITIALIZED = true; --We don't want an infinite loop, do we?
					RandomCompanion.Initialize();
					RandomCompanion.Pet();
				else
					RC_REINITIALIZED = nil;
				end
			else
				UIErrorsFrame:AddMessage("You have no vanity pets", 1.0, 0.0, 0.0, 53, 5);
			end
		end
	end
end

function RandomCompanion.PetDismiss()
	DismissCompanion("CRITTER");
end

function RandomCompanion.SetWeight(companion, value)
	if not RandomCompanion_settings then
		RandomCompanion_settings = {};
	end
	
	if not RandomCompanion_settings[GetRealmName()] then
		RandomCompanion_settings[GetRealmName()] = {};
	end
	
	if not RandomCompanion_settings[GetRealmName()][UnitName("player")] then
		RandomCompanion_settings[GetRealmName()][UnitName("player")] = {};
	end
	
	if not RandomCompanion_settings[GetRealmName()][UnitName("player")].weights then
		RandomCompanion_settings[GetRealmName()][UnitName("player")].weights = {};
	end
	
	RandomCompanion_settings[GetRealmName()][UnitName("player")].weights[companion] = value;
	debugmsg("RandomCompanion_settings["..GetRealmName().."]["..UnitName("player").."].weights["..companion.."] = "..RandomCompanion_settings[GetRealmName()][UnitName("player")].weights[companion], 1);
end

function RandomCompanion.GetWeight(companion)
	if RandomCompanion_settings[GetRealmName()][UnitName("player")].weights[companion] then
		return RandomCompanion_settings[GetRealmName()][UnitName("player")].weights[companion];
	else
		return 1;
	end
end

function RandomCompanion.GetActiveCompanion(type)
	local activecompanion = nil;
	for index = 1, GetNumCompanions(type) do
		local creatureid, name, spellid, icon, active = GetCompanionInfo(type, index);
		if active then
			debugmsg("Active " .. string.lower(type) .. " is index " .. index, 1);
			activecompanion = index;
		end
	end
	return activecompanion;
end

function RandomCompanion.SetActiveCompanions()
	local activecritter = RandomCompanion.GetActiveCompanion("CRITTER");
	if activecritter and activecritter ~= RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter then
		RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter = activecritter;
		debugmsg("Setting active critter to index " .. activecritter, 2);
	end
	if (RandomCompanion_settings.AutoRecall or RandomCompanion_settings.RandomRecall or RandomCompanion_settings.AutoDismiss) and RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter then
		RandomCompanion.OnUpdateTime = GetTime() + 2;
		RandomCompanion.EventFrame:SetScript("OnUpdate", RandomCompanion.OnUpdate);
	else
		RandomCompanion.EventFrame:SetScript("OnUpdate", nil);
	end
end

function RandomCompanion.RecallCritter()
	if RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter and RandomCompanion_settings.AutoRecall and RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter then
		local creatureid, name, spellid, icon, active = GetCompanionInfo("CRITTER", RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter);
		if not active then
			if (RandomCompanion_settings.AutoRecall or RandomCompanion_settings.RandomRecall or RandomCompanion_settings.AutoDismiss) and RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter then
				RandomCompanion.OnUpdateTime = GetTime() + 2;
				RandomCompanion.EventFrame:SetScript("OnUpdate", RandomCompanion.OnUpdate);
			else
				RandomCompanion.EventFrame:SetScript("OnUpdate", nil);
			end
			debugmsg("Automatically recalling critter from index " .. RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter .. " in 2 seconds", 4);
		end
	end
end

function RandomCompanion.OnUpdate()
	if RandomCompanion.OnUpdateTime == nil or GetTime() > RandomCompanion.OnUpdateTime then
		if (RandomCompanion_settings.AutoRecall or RandomCompanion_settings.RandomRecall or RandomCompanion_settings.AutoDismiss) and RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter then
			RandomCompanion.OnUpdateTime = GetTime() + 2;
			RandomCompanion.EventFrame:SetScript("OnUpdate", RandomCompanion.OnUpdate);
		else
			RandomCompanion.EventFrame:SetScript("OnUpdate", nil);
		end
		
		if (RandomCompanion.RecallTime == nil or GetTime() > RandomCompanion.RecallTime) and (RandomCompanion_settings.AutoRecall or RandomCompanion_settings.RandomRecall) and RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter then
			debugmsg("Checking to recall pets", 1);
			local creatureid, name, spellid, icon, active = GetCompanionInfo("CRITTER", RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter);
			RandomCompanion.RecallTime = GetTime() + 10;
			
			if not active then
				--Check to see if you're stealthed, prowling, or shadowmelded, because we don't want to break stealth to recall critters
				local eating = UnitBuff("player", RC_STRING_EATING);
				local drinking = UnitBuff("player", RC_STRING_DRINKING);
				local stealth = UnitBuff("player", RC_STRING_STEALTH);
				local prowl = UnitBuff("player", RC_STRING_PROWL);
				local shadowmeld = UnitBuff("player", RC_STRING_SHADOWMELD);
				local invisibility = UnitBuff("player", RC_STRING_INVISIBILITY);
				local combat = UnitAffectingCombat("player");
				local vehicle = UnitInVehicle("player");
				local casting, _, _, _, _, _ = UnitCastingInfo("player");
				local channeling, _, _, _, _, _ = UnitChannelInfo("player");
				local looting = GetNumLootItems();
				if looting == 0 and not eating and not drinking and not combat and not casting and not channeling and not IsMounted() and not vehicle and not stealth and not prowl and not shadowmeld then
					debugmsg("Recalling critter now", 4);
					if RandomCompanion_settings.RandomRecall then
						RandomCompanion.Pet();
					else
						CallCompanion("CRITTER", RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter);
					end
				else
					debugmsg("Not recalling critter because you are busy.", 1);
				end
			end
		end
		
		if RandomCompanion_settings.AutoDismiss and RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter then
			debugmsg("Checking to dismiss pets", 1);
			local creatureid, name, spellid, icon, active = GetCompanionInfo("CRITTER", RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter);
			if active then
				--Check to see if you're stealthed, prowling, or shadowmelded
				local stealth = UnitBuff("player", RC_STRING_STEALTH);
				local prowl = UnitBuff("player", RC_STRING_PROWL);
				local shadowmeld = UnitBuff("player", RC_STRING_SHADOWMELD);
				if (stealth or prowl or shadowmeld) and UnitIsPVP("player") then
					debugmsg("Auto-dismissing critter now", 2);
					local activecritter = RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter;
					DismissCompanion("CRITTER");
					--Restore RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter because DismissCompanion removes it
					RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter = activecritter;
				end
			end
		end
	end
end

function RandomCompanion.ScanForItem(item)
	debugmsg("Scanning for item: " .. item, 1);
	local founditem = false;
	
	for bagID = 0, 4 do
		debugmsg("Checking bag " .. bagID .. " with " .. GetContainerNumSlots(bagID) .. " slots", 1);
		for slotID = 1, GetContainerNumSlots(bagID) do
			local itemLink = GetContainerItemLink(bagID, slotID);
			if itemLink then
				local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(itemLink);
				--debugmsg(bagID .. "," .. slotID .. " contains " .. itemName);
				if string.lower(itemName) == string.lower(item) then
					debugmsg("Found item: " .. itemLink, 1);
					founditem = true;
					break;
				end
			end
		end
		if founditem then
			break;
		end
	end
	
	return founditem;
end

function RandomCompanion.Config()
	RandomCompanionConfig:Show();
end

--Hook into DismissCompanion() so that we don't auto-recall critters that were dismissed on purpose
local origDismissCompanion = DismissCompanion;
DismissCompanion = function(...)
	local companiontype = ...;
	
	if (string.lower(companiontype) == "critter") then
		RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter = nil;
		if (RandomCompanion_settings.AutoRecall or RandomCompanion_settings.RandomRecall or RandomCompanion_settings.AutoDismiss) and RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter then
			RandomCompanion.OnUpdateTime = GetTime() + 2;
			RandomCompanion.EventFrame:SetScript("OnUpdate", RandomCompanion.OnUpdate);
		else
			RandomCompanion.EventFrame:SetScript("OnUpdate", nil);
		end
		debugmsg("Critter manually dismissed. Will not auto-recall.", 4);
	end
	return origDismissCompanion(...);
end

function RandomCompanion.Slash(...)
	local parsedcmd = ...;
	local cmd = "";
	local options = "";
	
	debugmsg("cmd: " .. parsedcmd, 1);
	
	if (...) then
		--Parse any macro conditionals such as [modifier:ctrl] or [combat]
		parsedcmd = SecureCmdOptionParse(...);
	end
	
	debugmsg("parsed cmd: " .. parsedcmd, 1);
	
	if string.find(parsedcmd, " ") ~= nil then
		cmd = string.sub(parsedcmd, 0, string.find(parsedcmd, " ") - 1);
		options = string.sub(parsedcmd, string.find(parsedcmd, " ") + 1);
	elseif parsedcmd ~= nil then
		cmd = parsedcmd;
	end
	
	if string.lower(cmd) == "autorecall" then
		if (RandomCompanion_settings.AutoRecall or options == "off") and string.lower(options) ~= "on" then
			RandomCompanion_settings.AutoRecall = false;
			RandomCompanion_settings.RandomRecall = false;
			DEFAULT_CHAT_FRAME:AddMessage("RandomCompanion vanity pet recall disabled");
			if (RandomCompanion_settings.AutoRecall or RandomCompanion_settings.RandomRecall or RandomCompanion_settings.AutoDismiss) and RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter then
				RandomCompanion.OnUpdateTime = GetTime() + 2;
				RandomCompanion.EventFrame:SetScript("OnUpdate", RandomCompanion.OnUpdate);
			else
				RandomCompanion.EventFrame:SetScript("OnUpdate", nil);
			end
		else
			RandomCompanion_settings.AutoRecall = true;
			RandomCompanion_settings.RandomRecall = false;
			DEFAULT_CHAT_FRAME:AddMessage("RandomCompanion vanity pet recall enabled");
			if (RandomCompanion_settings.AutoRecall or RandomCompanion_settings.RandomRecall or RandomCompanion_settings.AutoDismiss) and RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter then
				RandomCompanion.OnUpdateTime = GetTime() + 2;
				RandomCompanion.EventFrame:SetScript("OnUpdate", RandomCompanion.OnUpdate);
			else
				RandomCompanion.EventFrame:SetScript("OnUpdate", nil);
			end
		end
	elseif string.lower(cmd) == "autodismiss" then
		if (RandomCompanion_settings.AutoDismiss or options == "off") and string.lower(options) ~= "on" then
			RandomCompanion_settings.AutoDismiss = false;
			DEFAULT_CHAT_FRAME:AddMessage("RandomCompanion vanity pet auto-dismiss disabled");
			if (RandomCompanion_settings.AutoRecall or RandomCompanion_settings.RandomRecall or RandomCompanion_settings.AutoDismiss) and RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter then
				RandomCompanion.OnUpdateTime = GetTime() + 2;
				RandomCompanion.EventFrame:SetScript("OnUpdate", RandomCompanion.OnUpdate);
			else
				RandomCompanion.EventFrame:SetScript("OnUpdate", nil);
			end
		else
			RandomCompanion_settings.AutoDismiss = true;
			DEFAULT_CHAT_FRAME:AddMessage("RandomCompanion vanity pet auto-dismiss enabled");
			if (RandomCompanion_settings.AutoRecall or RandomCompanion_settings.RandomRecall or RandomCompanion_settings.AutoDismiss) and RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter then
				RandomCompanion.OnUpdateTime = GetTime() + 2;
				RandomCompanion.EventFrame:SetScript("OnUpdate", RandomCompanion.OnUpdate);
			else
				RandomCompanion.EventFrame:SetScript("OnUpdate", nil);
			end
		end
	elseif string.lower(cmd) == "debug" then	
		if (options ~= "" and tonumber(options) == 0) or (RandomCompanion_settings.DEBUG and (tonumber(options) == nil or options == "")) then
			if (tonumber(options) ~= nil) then
				RandomCompanion_settings.DEBUGLevel = options;
			end
			RandomCompanion_settings.DEBUG = false;
			DEFAULT_CHAT_FRAME:AddMessage("RandomCompanion debug disabled");
		else
			if (tonumber(options) ~= nil) then
				RandomCompanion_settings.DEBUGLevel = options;
			end
			if tonumber(RandomCompanion_settings.DEBUGLevel) < 1 or tonumber(RandomCompanion_settings.DEBUGLevel) == nil then
				RandomCompanion_settings.DEBUGLevel = 1;
			end
			RandomCompanion_settings.DEBUG = true;
			DEFAULT_CHAT_FRAME:AddMessage("RandomCompanion debug enabled - Level " .. RandomCompanion_settings.DEBUGLevel);
		end
	elseif string.lower(cmd) == "mount" then
		RandomCompanion.Mount(options);
	elseif string.lower(cmd) == "pet" then
		RandomCompanion.Pet(options);
	elseif string.lower(cmd) == "petdismiss" or string.lower(cmd) == "dismiss" then
		RandomCompanion.PetDismiss(options);
	elseif string.lower(cmd) == "randomrecall" then
		if (RandomCompanion_settings.RandomRecall or options == "off") and string.lower(options) ~= "on" then
			RandomCompanion_settings.AutoRecall = false;
			RandomCompanion_settings.RandomRecall = false;
			DEFAULT_CHAT_FRAME:AddMessage("RandomCompanion random vanity pet recall disabled");
		else
			RandomCompanion_settings.AutoRecall = false;
			RandomCompanion_settings.RandomRecall = true;
			DEFAULT_CHAT_FRAME:AddMessage("RandomCompanion random vanity pet recall enabled");
		end
	elseif string.lower(cmd) == "reload" then
		RandomCompanion.Initialize(options);
	elseif string.lower(cmd) == "status" then
		RandomCompanion.Status(options);
	else --Show help
		DEFAULT_CHAT_FRAME:AddMessage("|cFFFFFF00 RandomCompanion version " .. VERSION);
		DEFAULT_CHAT_FRAME:AddMessage("|cFFFFFF00 Bind a key from the keybinding menu or...");
		DEFAULT_CHAT_FRAME:AddMessage("|cFFFFFF00 \"/rc mount\" to choose a random mount");
		DEFAULT_CHAT_FRAME:AddMessage("|cFFFFFF00 \"/rc mount ground\" to choose a random ground mount in a flyable zone");
		DEFAULT_CHAT_FRAME:AddMessage("|cFFFFFF00 \"/rc pet\" to choose a random vanity pet");
		DEFAULT_CHAT_FRAME:AddMessage("|cFFFFFF00 \"/rc autorecall\" to toggle automatically recalling your vanity pet after resurrecting, changing zones, or taking flight paths");
		DEFAULT_CHAT_FRAME:AddMessage("|cFFFFFF00 \"/rc randomrecall\" to toggle recalling a random vanity pet after resurrecting, changing zones, or taking flight paths");
		DEFAULT_CHAT_FRAME:AddMessage("|cFFFFFF00 \"/rc dismiss\" to dismiss your current vanity pet");
		DEFAULT_CHAT_FRAME:AddMessage("|cFFFFFF00 \"/rc autodismiss\" to toggle automatically dismissing your vanity pet when you are stealthed and flagged for PVP");
	end
end

SlashCmdList["RANDOMMOUNT"] = RandomCompanion.Mount;
SLASH_RANDOMMOUNT1 = "/randommount";
SLASH_RANDOMMOUNT2 = "/rmount";
SLASH_RANDOMMOUNT3 = "/rmt";

SlashCmdList["RANDOMPET"] = RandomCompanion.Pet;
SLASH_RANDOMPET1 = "/randompet";
SLASH_RANDOMPET2 = "/rpet";
SLASH_RANDOMPET3 = "/rpt";

SlashCmdList["PETDISMISS"] = RandomCompanion.PetDismiss;
SLASH_PETDISMISS1 = "/petdismiss";
SLASH_PETDISMISS2 = "/pdismiss";

SlashCmdList["RCSTATUS"] = RandomCompanion.Status;
SLASH_RCSTATUS1 = "/rcstatus";

SlashCmdList["RCDEBUG"] = RandomCompanion.Debug;
SLASH_RCDEBUG1 = "/rcdebug";

SlashCmdList["RCRELOAD"] = RandomCompanion.Initialize;
SLASH_RCRELOAD1 = "/rcreload";

SlashCmdList["RC"] = RandomCompanion.Slash;
SLASH_RC1 = "/rc";
SLASH_RC2 = "/randomcompanion";

RandomCompanion.EventFrame = CreateFrame("Frame", "RandomCompanionFrame");
RandomCompanion.EventFrame:SetScript("OnEvent", RandomCompanion.Event);
RandomCompanion.EventFrame:RegisterEvent("COMPANION_LEARNED");
RandomCompanion.EventFrame:RegisterEvent("COMPANION_UPDATE");
RandomCompanion.EventFrame:RegisterEvent("PLAYER_LOGIN");
RandomCompanion.EventFrame:RegisterEvent("PLAYER_CONTROL_GAINED");
RandomCompanion.EventFrame:RegisterEvent("PLAYER_UNGHOST");
RandomCompanion.EventFrame:RegisterEvent("PLAYER_ALIVE");
RandomCompanion.EventFrame:RegisterEvent("PLAYER_ENTERING_WORLD");
